class DruidEnterprise(DataSource):
    def __init__(self, host=None,
                 port='8888', path='/druid/v2/sql', scheme='http',
                 begin_time=None, topic="ssh", max_records=None, table_name="ssh"):
        super().__init__()
        self._host = host
        self._port = port
        self._path = path
        self._scheme = scheme
        self._begin_time = begin_time
        self._topic = topic
        self._max_records = max_records
        self._table_name = table_name

    def open(self):
        from pydruid.db import connect
        conn = connect(host=self._host, port=self._port, path=self._path, scheme=self._scheme)
        self._conn
        self._curs = self._conn.cursor()

        if self._begin_time:
            time_field="where __time > "
            timestamp = parser.parse(self._begin_time).timestamp()
            (tm_year, tm_mon, tm_mday, tm_hour, tm_min,
             tm_sec, tm_wday, tm_yday, tm_isdst) = time.gmtime(timestamp)
            formated_time = "'%04d-%02d-%02d %02d:%02d:%02d'" % (tm_year, tm_mon, tm_mday, tm_hour, tm_min, tm_sec)
            self._curs.execute("select * from %s where __time > %s limit %d" %
                               (self._table_name, formated_time, args.max_records))
        else:
            self._curs.execute("select * from %s %s limit %d" % (formated_time, args.max_records))

        return self

    def __iter__(self):
        return self

    def __next__(self):
        return(next(self._curs))
        # try:
        #     for row in self._curs:
        #         yield row 
        # except StopIteration as e:
        #     return




import sys
import re

from gawseed.threatfeed.search.re import RESearch

class HTTPSearch(RESearch):
    """Matches a regexp threat against a host/uri combo data field (aka bro log)"""
    def __init__(self, search_list, data_iterator, binary_search, conf={}):
        super().__init__(search_list, data_iterator, binary_search, conf)
    
    def search(self, row):
        # XXX: currently assumes a split host/uri scheme like bro.  Need to make this generic
        if b'host' not in row or b'uri' not in row or row[b'host'] is None or row[b'uri'] is None:
            return None
        if self._binary_search:
            url = "http://" + row[b'host'].decode() + "/" + row[b'uri'].decode()
        else:
            url = "http://" + row[b'host'].decode() + "/" + row[b'uri'].decode()

        return self.search_one(url)




import pyfsdb
from . import DataSource

class FsdbDataSource(DataSource):
    """Loads data from a tab-separated FSDB formatted file (see pyfsdb)"""
    def __init__(self, conf):
        super().__init__(conf)
        
        self._file_handle = self.config('file_handle',
                                        help="A python3 opened file handle for the BRO data to be streamed")
        self._file = self.config('file',
                                 help="The file name to read the bro data stream from")

    def initialize(self):
        if not self._file_handle and not self._file:
            self.config_error("either file_handle or file is required for the %s module" % (type(self)))

    def open(self):
        self._fh = pyfsdb.Fsdb(file_handle=self._file_handle, filename=self._file,
                               return_type=pyfsdb.RETURN_AS_DICTIONARY)

    def __iter__(self):
        return self

    def __next__(self):
        return (next(self._fh))




class Config():
    def __init__(self, config={}):
        self._config = config

    def initialize(self):
        """Overridable function for doing things beyond config copying in __init__"""
        pass

    def set_defaults(self, defaults={}):
        for default in defaults:
            if default not in self_.config:
                self._config = defaults[default]

    def require(self, requirements):
        if type(requirements) != list:
            requirements = [requirements]
        if 'dump_config' in self._config:
            print("      # required: " + str(requirements))
            return
        for requirement in requirements:
            if requirement not in self._config:
                self.config_error("'%s' is a requirement argument for %s" % (requirement, type(self)))

    def config(self, name, default=None, help=None):
        if 'dump_config' in self._config:
            if help:
                print("      # %s: %s" % (name, help))
            print("      %s: %s" % (name, default))
            return
        if name in self._config:
            return self._config[name]
        return default

    def config_error(self, msg):
        raise ValueError(msg)



import sys
import re

from gawseed.threatfeed.search.re import RESearch

class DNSSearch(RESearch):
    """Searches for a threat regexp in DNS query logs"""
    def __init__(self, search_list, data_iterator, binary_search, conf={}):
        super().__init__(search_list, data_iterator, binary_search, conf)
        self._key = self.config('key', 'query',
                                help="The key of the field in the data stream to search for matches") # assumes bro
        if binary_search:
            self._key = bytes(self._key, 'utf-8')

